import { 
  select, 
  tsv, 
  json, 
  geoPath, 
  geoMercator, 
  zoom, 
  event 
} from 'd3';
import { feature } from 'topojson';

// Convert the JSON array into an object that has the Country as the key and the Country object as the value
function toObject(arr) {
  let res = {};
  for (let i = 0; i < arr.length; ++i) {
    let key = arr[i].Country;
   	res[key] = arr[i];
  }
 
  return res;
};

// Used to print the tooltip info if the country is included in the WHO suicide report,
// otherwise just prints the country name and unknowns for the rest of the values
function printHoverInfo(obj, isObj) {
 	let retString = '';
  if (isObj) {
    retString = 'Country: ' + obj.Country + '\n'
              + 'Happiness Rank: ' + obj['Happiness Rank'] + '\n'
              + 'Happiness Score: ' + obj['Happiness Score'] + '\n'
              + 'Suicide Mortality Rate (Per 100,000): ' + obj['Suicide Mortality Rate (Per 100,000)'] + '\n'
              + 'Universal Health Care: ' + (obj['Universal Health Care'] == 1 ? 'Yes' : 'No');
  } else {
    let name = obj;
    retString = 'Country: ' + name + '\n'
              + 'Happiness Rank: Unknown\n'
              + 'Happiness Score: Unknown\n'
              + 'Suicide Mortality Rate (Per 100,000): Unknown\n'
              + 'Universal Health Care: Unknown';
  }
  return retString;
}

const svg = select('svg');

const projection = geoMercator();
const pathGenerator = geoPath().projection(projection);

const g = svg.append('g');

g.append('path')
  .attr('class', 'sphere')
  .attr('d', pathGenerator({type: 'Sphere'}));

svg.call(zoom().on('zoom', () => {
	g.attr('transform', event.transform);
}));

Promise.all([
	json('https://cdn.jsdelivr.net/npm/world-atlas@2.0.2/countries-50m.json'),
  json('https://raw.githubusercontent.com/cse442-20f/FP-Healthcare-and-The-Will-To-Live/master/docs/Happiness_Dataset_2016.json?token=AGNSN3FIPK6MSLED3KVPMQC7Z3TFC')
]).then(([topoJSONdata, happinessJSON]) => {
  const countries = feature(topoJSONdata, topoJSONdata.objects.countries);
	
  let res = toObject(happinessJSON);
  console.log(res);
  g.selectAll('path').data(countries.features)
    .enter().append('path')
      .attr('class', 'country')
      .attr('d', pathGenerator)
    .append('title')
      .text(d => {
    		let obj = res[d.properties.name];
    		if (obj) {
          return printHoverInfo(obj, true);
        } else {
         	return printHoverInfo(d.properties.name, false);
        }
    		//console.log(d.properties);
  });
});